@@ -45,7 +45,8 @@ class ForecasterAutoreg(ForecasterBase):
     lags : int, list, 1d numpy ndarray, range
         Lags used as predictors. Index starts at 1, so lag 1 is equal to t-1.
             `int`: include lags from 1 to `lags` (included).
-            `list`, `numpy ndarray` or `range`: include only lags present in `lags`.
+            `list`, `numpy ndarray` or `range`: include only lags present in `lags`,
+            all elements must be int.
 
     
     Attributes
@@ -134,6 +135,11 @@ class ForecasterAutoreg(ForecasterBase):
             
         if isinstance(lags, (list, range, np.ndarray)) and min(lags) < 1:
             raise Exception('Minimum value of lags allowed is 1.')
+
+        if isinstance(lags, (list, np.ndarray)):
+            for lag in lags:
+                if not isinstance(lag, (int, np.int64, np.int32)):
+                    raise Exception('Values in lags must be int.')
             
         if isinstance(lags, int):
             self.lags = np.arange(lags) + 1
@@ -327,7 +333,6 @@ class ForecasterAutoreg(ForecasterBase):
         self.exog_col_names       = None
         self.X_train_col_names    = None
         self.in_sample_residuals  = None
-        self.out_sample_residuals = None
         self.fitted               = False
         self.training_range       = None
         
@@ -358,7 +363,8 @@ class ForecasterAutoreg(ForecasterBase):
             residuals = y_train - self.regressor.predict(X_train.to_numpy())
         if len(residuals) > 1000:
             # Only up to 1000 residuals are stored
-            residuals = np.random.choice(a=residuals, size=1000, replace=False)                                              
+            rng = np.random.default_rng(seed=123)
+            residuals = rng.choice(a=residuals, size=1000, replace=False)                                              
         self.in_sample_residuals = residuals
         
         # The last time window of training data is stored so that lags needed as
@@ -589,7 +595,7 @@ class ForecasterAutoreg(ForecasterBase):
                 exog_boot = exog.copy()
             else:
                 exog_boot = None
-                
+ 
             if in_sample_residuals:
                 residuals = self.in_sample_residuals
             else:
@@ -665,7 +671,7 @@ class ForecasterAutoreg(ForecasterBase):
             Number of bootstrapping iterations used to estimate prediction
             intervals.
 
-        random_state: int
+        random_state: int, default 123
             Sets a seed to the random generator, so that boot intervals are always 
             deterministic.
             
@@ -785,7 +791,7 @@ class ForecasterAutoreg(ForecasterBase):
         self.regressor.set_params(**params)
         
         
-    def set_lags(self, lags: int) -> None:
+    def set_lags(self, lags: Union[int, list, np.ndarray, range]) -> None:
         '''      
         Set new value to the attribute `lags`.
         Attributes `max_lag` and `window_size` are also updated.
@@ -793,13 +799,13 @@ class ForecasterAutoreg(ForecasterBase):
         Parameters
         ----------
         lags : int, list, 1D np.array, range
-        Lags used as predictors. Index starts at 1, so lag 1 is equal to t-1.
-            `int`: include lags from 1 to `lags`.
-            `list` or `np.array`: include only lags present in `lags`.
+            Lags used as predictors. Index starts at 1, so lag 1 is equal to t-1.
+                `int`: include lags from 1 to `lags`.
+                `list` or `np.array`: include only lags present in `lags`.
 
         Returns 
         -------
-        self
+        None
         
         '''
         
@@ -825,7 +831,7 @@ class ForecasterAutoreg(ForecasterBase):
         self.window_size = max(self.lags)
         
         
-    def set_out_sample_residuals(self, residuals: np.ndarray, append: bool=True)-> None:
+    def set_out_sample_residuals(self, residuals: pd.Series, append: bool=True)-> None:
         '''
         Set new values to the attribute `out_sample_residuals`. Out of sample
         residuals are meant to be calculated using observations that did not
@@ -833,7 +839,7 @@ class ForecasterAutoreg(ForecasterBase):
         
         Parameters
         ----------
-        params : 1D np.ndarray
+        residuals : pd.Series
             Values of residuals. If len(residuals) > 1000, only a random sample
             of 1000 values are stored.
             
@@ -843,35 +849,36 @@ class ForecasterAutoreg(ForecasterBase):
             reached, no more values are appended. If False, `out_sample_residuals`
             is overwritten with the new residuals.
             
-
         Returns 
         -------
         self
-        
         '''
-        if not isinstance(residuals, np.ndarray):
+
+        if not isinstance(residuals, pd.Series):
             raise Exception(
-                f"`residuals` argument must be `1D np.ndarray`. Got {type(residuals)}"
+                f"`residuals` argument must be `pd.Series`. Got {type(residuals)}"
             )
-            
+
         if len(residuals) > 1000:
-            residuals = np.random.choice(a=residuals, size=1000, replace=False)
-                                 
-        if not append or self.out_sample_residuals is None:
-            self.out_sample_residuals = residuals
-        else:
+            rng = np.random.default_rng(seed=123)
+            residuals = rng.choice(a=residuals, size=1000, replace=False)
+            residuals = pd.Series(residuals)   
+      
+        if append and self.out_sample_residuals is not None:
             free_space = max(0, 1000 - len(self.out_sample_residuals))
             if len(residuals) < free_space:
-                self.out_sample_residuals = np.hstack((
-                                                self.out_sample_residuals,
-                                                residuals
-                                            ))
+                residuals = np.hstack((
+                                self.out_sample_residuals,
+                                residuals
+                            ))
             else:
-                self.out_sample_residuals = np.hstack((
-                                                self.out_sample_residuals,
-                                                residuals[:free_space]
-                                            ))
-        
+                residuals = np.hstack((
+                                self.out_sample_residuals,
+                                residuals[:free_space]
+                            ))
+
+        self.out_sample_residuals = pd.Series(residuals)
+
 
     def get_coef(self) -> pd.DataFrame:
         '''      
@@ -889,7 +896,11 @@ class ForecasterAutoreg(ForecasterBase):
             Value of the coefficients associated with each predictor.
         
         '''
-        
+        warnings.warn(
+            f'This method was deprecated in version 0.4.3 in favor of the get_feature_importance. '
+            f'This method will be removed in 0.4.4', DeprecationWarning
+        )
+
         if isinstance(self.regressor, sklearn.pipeline.Pipeline):
             estimator = self.regressor[-1]
         else:
@@ -916,13 +927,13 @@ class ForecasterAutoreg(ForecasterBase):
         '''      
         Return feature importance of the regressor stored in the
         forecaster. Only valid when regressor stores internally the feature
-        importance in the attribute `feature_importances_`.
+        importance in the attribute `feature_importances_` or `coef_`.
 
         Parameters
         ----------
         self
 
-        Returns 
+        Returns
         -------
         feature_importance : pandas DataFrame
             Feature importance associated with each predictor.
@@ -938,13 +949,20 @@ class ForecasterAutoreg(ForecasterBase):
                                     'feature': self.X_train_col_names,
                                     'importance' : estimator.feature_importances_
                                 })
-        except:
-            warnings.warn(
-                f"Impossible to access feature importance for regressor of type {type(estimator)}. "
-                f"This method is only valid when the regressor stores internally "
-                f" the feature importance in the attribute `feature_importances_`."
-            )
+        except:   
+            try:
+                feature_importance = pd.DataFrame({
+                                        'feature': self.X_train_col_names,
+                                        'importance' : estimator.coef_
+                                    })
+            except:
+                warnings.warn(
+                    f"Impossible to access feature importance for regressor of type {type(estimator)}. "
+                    f"This method is only valid when the regressor stores internally "
+                    f"the feature importance in the attribute `feature_importances_` "
+                    f"or `coef_`."
+                )
+
+                feature_importance = None
 
-            feature_importance = None
-        
         return feature_importance
