@@ -311,7 +311,6 @@ class ForecasterAutoregCustom(ForecasterBase):
         self.exog_col_names       = None
         self.in_sample_residuals  = None
         self.X_train_col_names    = None
-        self.out_sample_residuals = None
         self.fitted               = False
         self.training_range       = None
         
@@ -342,7 +341,8 @@ class ForecasterAutoregCustom(ForecasterBase):
             residuals = y_train - self.regressor.predict(X_train.to_numpy())
         if len(residuals) > 1000:
             # Only up to 1000 residuals are stored
-            residuals = np.random.choice(a=residuals, size=1000, replace=False)                                              
+            rng = np.random.default_rng(seed=123)
+            residuals = rng.choice(a=residuals, size=1000, replace=False)                                              
         self.in_sample_residuals = residuals
         
         # The last time window of training data is stored so that predictors in
@@ -655,7 +655,7 @@ class ForecasterAutoregCustom(ForecasterBase):
             Number of bootstrapping iterations used to estimate prediction
             intervals.
 
-        random_state: int
+        random_state: int, default 123
             Sets a seed to the random generator, so that boot intervals are always 
             deterministic.
             
@@ -776,7 +776,7 @@ class ForecasterAutoregCustom(ForecasterBase):
         self.regressor.set_params(**params)
         
     
-    def set_out_sample_residuals(self, residuals: np.ndarray, append: bool=True)-> None:
+    def set_out_sample_residuals(self, residuals: pd.Series, append: bool=True)-> None:
         '''
         Set new values to the attribute `out_sample_residuals`. Out of sample
         residuals are meant to be calculated using observations that did not
@@ -784,7 +784,7 @@ class ForecasterAutoregCustom(ForecasterBase):
         
         Parameters
         ----------
-        params : 1D np.ndarray
+        residuals : pd.Series
             Values of residuals. If len(residuals) > 1000, only a random sample
             of 1000 values are stored.
             
@@ -794,34 +794,35 @@ class ForecasterAutoregCustom(ForecasterBase):
             reached, no more values are appended. If False, `out_sample_residuals`
             is overwritten with the new residuals.
             
-
         Returns 
         -------
         self
-        
         '''
-        if not isinstance(residuals, np.ndarray):
+
+        if not isinstance(residuals, pd.Series):
             raise Exception(
-                f"`residuals` argument must be `1D np.ndarray`. Got {type(residuals)}"
+                f"`residuals` argument must be `pd.Series`. Got {type(residuals)}"
             )
             
         if len(residuals) > 1000:
-            residuals = np.random.choice(a=residuals, size=1000, replace=False)
-                                 
-        if not append or self.out_sample_residuals is None:
-            self.out_sample_residuals = residuals
-        else:
+            rng = np.random.default_rng(seed=123)
+            residuals = rng.choice(a=residuals, size=1000, replace=False)
+            residuals = pd.Series(residuals)   
+    
+        if append and self.out_sample_residuals is not None:
             free_space = max(0, 1000 - len(self.out_sample_residuals))
             if len(residuals) < free_space:
-                self.out_sample_residuals = np.hstack((
-                                                self.out_sample_residuals,
-                                                residuals
-                                            ))
+                residuals = np.hstack((
+                                self.out_sample_residuals,
+                                residuals
+                            ))
             else:
-                self.out_sample_residuals = np.hstack((
-                                                self.out_sample_residuals,
-                                                residuals[:free_space]
-                                            ))
+                residuals = np.hstack((
+                                self.out_sample_residuals,
+                                residuals[:free_space]
+                            ))
+
+        self.out_sample_residuals = pd.Series(residuals)
                 
 
     def get_coef(self) -> pd.DataFrame:
@@ -840,6 +841,10 @@ class ForecasterAutoregCustom(ForecasterBase):
             Value of the coefficients associated with each predictor.
         
         '''
+        warnings.warn(
+            f'This method was deprecated in version 0.4.3 in favor of the get_feature_importance. '
+            f'This method will be removed in 0.4.4', DeprecationWarning
+        )
         
         if isinstance(self.regressor, sklearn.pipeline.Pipeline):
             estimator = self.regressor[-1]
@@ -867,13 +872,13 @@ class ForecasterAutoregCustom(ForecasterBase):
         '''      
         Return feature importance of the regressor stored in the
         forecaster. Only valid when regressor stores internally the feature
-        importance in the attribute `feature_importances_`.
+        importance in the attribute `feature_importances_` or `coef_`.
 
         Parameters
         ----------
         self
 
-        Returns 
+        Returns
         -------
         feature_importance : pandas DataFrame
             Feature importance associated with each predictor.
@@ -889,13 +894,20 @@ class ForecasterAutoregCustom(ForecasterBase):
                                     'feature': self.X_train_col_names,
                                     'importance' : estimator.feature_importances_
                                 })
-        except:
-            warnings.warn(
-                f"Impossible to access feature importance for regressor of type {type(estimator)}. "
-                f"This method is only valid when the regressor stores internally "
-                f" the feature importance in the attribute `feature_importances_`."
-            )
+        except:   
+            try:
+                feature_importance = pd.DataFrame({
+                                        'feature': self.X_train_col_names,
+                                        'importance' : estimator.coef_
+                                    })
+            except:
+                warnings.warn(
+                    f"Impossible to access feature importance for regressor of type {type(estimator)}. "
+                    f"This method is only valid when the regressor stores internally "
+                    f"the feature importance in the attribute `feature_importances_` "
+                    f"or `coef_`."
+                )
+
+                feature_importance = None
 
-            feature_importance = None
-        
         return feature_importance
