@@ -150,6 +150,11 @@ class ForecasterAutoregMultiOutput(ForecasterBase):
             
         if isinstance(lags, (list, range, np.ndarray)) and min(lags) < 1:
             raise Exception('Minimum value of lags allowed is 1')
+
+        if isinstance(lags, (list, np.ndarray)):
+            for lag in lags:
+                if not isinstance(lag, (int, np.int64, np.int32)):
+                    raise Exception('Values in lags must be int.')
             
         if isinstance(lags, int):
             self.lags = np.arange(lags) + 1
@@ -572,7 +577,7 @@ class ForecasterAutoregMultiOutput(ForecasterBase):
         
         
         
-    def set_lags(self, lags: int) -> None:
+    def set_lags(self, lags: Union[int, list, np.ndarray, range]) -> None:
         '''      
         Set new value to the attribute `lags`.
         Attributes `max_lag` and `window_size` are also updated.
@@ -580,9 +585,9 @@ class ForecasterAutoregMultiOutput(ForecasterBase):
         Parameters
         ----------
         lags : int, list, 1D np.array, range
-        Lags used as predictors. Index starts at 1, so lag 1 is equal to t-1.
-            `int`: include lags from 1 to `lags`.
-            `list` or `np.array`: include only lags present in `lags`.
+            Lags used as predictors. Index starts at 1, so lag 1 is equal to t-1.
+                `int`: include lags from 1 to `lags`.
+                `list` or `np.array`: include only lags present in `lags`.
 
         Returns 
         -------
@@ -626,7 +631,7 @@ class ForecasterAutoregMultiOutput(ForecasterBase):
         ----------
         step : int
             Model from which retrieve information (a separate model is created for
-            each forecast time step).
+            each forecast time step). First step is 1.
 
         Returns 
         -------
@@ -634,20 +639,36 @@ class ForecasterAutoregMultiOutput(ForecasterBase):
             Value of the coefficients associated with each predictor.
         
         '''
-        
+        warnings.warn(
+            f'This method was deprecated in version 0.4.3 in favor of the get_feature_importance. '
+            f'This method will be removed in 0.4.4', DeprecationWarning
+        )
+
         if step > self.steps:
             raise Exception(
                 f"Forecaster trained for {self.steps} steps. Got step={step}."
             )
+        if step < 1:
+            raise Exception("Minimum step is 1.")
+        
+        # Stored regressors start at index 0
+        step = step - 1
             
         if isinstance(self.regressor, sklearn.pipeline.Pipeline):
-            estimator = self.regressors_[step-1][-1]
+            estimator = self.regressors_[step][-1]
         else:
-            estimator = self.regressors_[step-1]
+            estimator = self.regressors_[step]
 
         try:
+            idx_columns_lags = np.arange(len(self.lags))
+            idx_columns_exog = np.array([], dtype=int)
+            if self.included_exog:
+                idx_columns_exog = np.arange(len(self.X_train_col_names))[len(self.lags) + step::self.steps]
+            idx_columns = np.hstack((idx_columns_lags, idx_columns_exog))
+            feature_names = [self.X_train_col_names[i] for i in idx_columns]
+            feature_names = [name.replace(f"_step_{step+1}", "") for name in feature_names]
             coef = pd.DataFrame({
-                        'feature': self.X_train_col_names,
+                        'feature': feature_names,
                         'coef' : estimator.coef_
                    })
         except:
@@ -668,7 +689,7 @@ class ForecasterAutoregMultiOutput(ForecasterBase):
         the forecaster for a specific step. Since a separate model is created for
         each forecast time step, it is necessary to select the model from which
         retrieve information.
-        
+
         Only valid when the forecaster has been trained using 
         `GradientBoostingRegressor`, `RandomForestRegressor` or 
         `HistGradientBoostingRegressor` as regressor.
@@ -677,36 +698,61 @@ class ForecasterAutoregMultiOutput(ForecasterBase):
         ----------
         step : int
             Model from which retrieve information (a separate model is created for
-            each forecast time step).
+            each forecast time step). First step is 1.
 
         Returns 
         -------
         feature_importance : pandas DataFrame
             Impurity-based feature importance associated with each predictor.
         '''
-        
         if step > self.steps:
             raise Exception(
                 f"Forecaster trained for {self.steps} steps. Got step={step}."
             )
-        
+        if step < 1:
+            raise Exception("Minimum step is 1.")
+
+        # Stored regressors start at index 0
+        step = step - 1
+
         if isinstance(self.regressor, sklearn.pipeline.Pipeline):
-            estimator = self.regressors_[step-1][-1]
+            estimator = self.regressors_[step][-1]
         else:
-            estimator = self.regressors_[step-1]
+            estimator = self.regressors_[step]
 
         try:
+            idx_columns_lags = np.arange(len(self.lags))
+            idx_columns_exog = np.array([], dtype=int)
+            if self.included_exog:
+                idx_columns_exog = np.arange(len(self.X_train_col_names))[len(self.lags) + step::self.steps]
+            idx_columns = np.hstack((idx_columns_lags, idx_columns_exog))
+            feature_names = [self.X_train_col_names[i] for i in idx_columns]
+            feature_names = [name.replace(f"_step_{step+1}", "") for name in feature_names]
             feature_importance = pd.DataFrame({
-                                    'feature': self.X_train_col_names,
+                                    'feature': feature_names,
                                     'importance' : estimator.feature_importances_
                                 })
-        except:
-            warnings.warn(
-                f"Impossible to access feature importance for regressor of type {type(estimator)}. "
-                f"This method is only valid when the regressor stores internally "
-                f" the feature importance in the attribute `feature_importances_`."
-            )
+        except:   
+            try:
+                idx_columns_lags = np.arange(len(self.lags))
+                idx_columns_exog = np.array([], dtype=int)
+                if self.included_exog:
+                    idx_columns_exog = np.arange(len(self.X_train_col_names))[len(self.lags) + step::self.steps]
+                idx_columns = np.hstack((idx_columns_lags, idx_columns_exog))
+                feature_names = [self.X_train_col_names[i] for i in idx_columns]
+                feature_names = [name.replace(f"_step_{step+1}", "") for name in feature_names]
+                feature_importance = pd.DataFrame({
+                                        'feature': feature_names,
+                                        'importance' : estimator.coef_
+                                    })
+            except:
+                warnings.warn(
+                    f"Impossible to access feature importance for regressor of type {type(estimator)}. "
+                    f"This method is only valid when the regressor stores internally "
+                    f"the feature importance in the attribute `feature_importances_` "
+                    f"or `coef_`."
+                )
 
-            feature_importance = None
+                feature_importance = None
 
         return feature_importance
